/***************************************************************************
 *   Copyright (C) 2004 by Joseph Wenninger                                *
 *   jowenn@kde.org                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "connecting.h"
#include <klocale.h>
#include <knxconnection.h>
#include <kpushbutton.h>
#include <kled.h>
#include <qtimer.h>
#include <qlabel.h>
#include <kmessagebox.h>

ConnectingDialog::ConnectingDialog(KNXConnection *connection,const QString& connectionname): Connecting_Base() { 
	setCaption(connectionname+" - kNX");
	connectionlabel->setText(i18n("Connection: %1").arg(connectionname));
	m_connection=connection;
	connect(cancelbutton,SIGNAL(clicked()),this,SLOT(slotCancel()));
	m_ledTimer=new QTimer(this);
	connect(m_ledTimer,SIGNAL(timeout()),this,SLOT(ledAnim()));
	m_ledState=0;
	m_ledTimer->start(250);
	setFixedSize(sizeHint());
}


void ConnectingDialog::clearConnection() {
	m_connection=0;
}

ConnectingDialog::~ConnectingDialog() {
}


void ConnectingDialog::setActivity(const QString &act) {
	label_status->setText(act);
}

void ConnectingDialog::slotCancel() {
	close();
}

void ConnectingDialog::closeEvent ( QCloseEvent * e ) {
	Connecting_Base::closeEvent(e);
	if (m_connection) m_connection->abortConnecting();
	deleteLater();
}

void ConnectingDialog::timeout() {
	error();
	KMessageBox::sorry(this,i18n("<qt>Connection to NX server could not be established.<BR>Connection timed out.</qt>"));
	close();
}

void ConnectingDialog::accessDenied(bool nx,const QString &hostName) {
	error();
	if (!nx)
		KMessageBox::sorry(this,i18n("<qt>Remote host (%1) could not be resolved<BR><B>or</B><BR>you are not allowed to access it.</qt>").arg(hostName));
	else
		KMessageBox::sorry(this,i18n("<qt>The NX server (%1) rejected your login information.</qt>").arg(hostName));
	close();
}

void ConnectingDialog::badServerKey(const QString &hostName) {
	error();
		KMessageBox::sorry(this,i18n("<qt>It was not possible to ensure that the requested server <B>(%1)</B> is really the one it claims to be.</qt>").arg(hostName));
	close();
}

void ConnectingDialog::unknownError() {
	error();
	KMessageBox::sorry(this,i18n("<qt>An unexpected error occurred while connecting.</qt>"));
	close();
}

void ConnectingDialog::noSSH(const QString &hostName) {
	error();
	KMessageBox::sorry(this,i18n("<qt>The remote host (%1) could not be reached, or it does not have a running SSH daemon.</qt>").arg(hostName));
	close();	
}

void ConnectingDialog::error() {
	m_ledTimer->stop();

	Led1->setColor(QColor(0xff,0,0));
	Led2->setColor(QColor(0xff,0,0));
	Led3->setColor(QColor(0xff,0,0));
	Led4->setColor(QColor(0xff,0,0));

	Led1->setState(KLed::On);
	Led2->setState(KLed::On);
	Led3->setState(KLed::On);
	Led4->setState(KLed::On);
}

void ConnectingDialog::ledAnim() {
	switch (m_ledState) {
		case 0:
				Led3->setState(KLed::Off);
				Led1->setState(KLed::On);
			break;
		case 1:
				Led4->setState(KLed::Off);
				Led2->setState(KLed::On);
			break;
		case 2:
				Led1->setState(KLed::Off);
				Led3->setState(KLed::On);
			break;
		case 3:
				Led2->setState(KLed::Off);
				Led4->setState(KLed::On);
			break;
	}
	m_ledState=(m_ledState+1)%4;
}

#include "connecting.moc"
