/***************************************************************************
 *   Copyright (C) 2004 by Joseph Wenninger                                *
 *   jowenn@kde.org                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef _KNX_CONNECTION_
#define _KNX_CONNECTION_

#include <qobject.h>
#include <qguardedptr.h>
#include <connecting.h>
#include <qdom.h>


#warning remove inlines

class KNXConnection : public QObject {
	Q_OBJECT
public:	
	KNXConnection();
	virtual ~KNXConnection();
	bool doConnect(const QString&);

	void setSSHConnection(const QString& user, const QString &host=QString::null, const QString &port=QString::null);
	void setNXLogin(const QString& password, const QString& username);

	void setLoginName(const QString& loginname);
	void setLoginPassword(const QString& loginpassword);
	void setRemoteHost(const QString& remoteHost);
	void setSSHUser(const QString& sshUser);
	void setSettings(const QString&);
	void setDataRoot(const QString&);
	void abortConnecting();
	void load(const QString& connectionFile);

private:
	enum State {
		INITIALIZATION=1,
		PREAMBLE_SHELL,
		PREAMBLE_PASSWORD,
		LOGIN,
		LOGIN_NAME,
		LOGIN_PASSWORD,
		STARTSESSION,
		GETID,
		GETDISPLAY,
		GETCOOKIE,
		SESSION,
		SESSIONRUNNING,
		ERROR
		
	} m_state;

	class KProcess *m_sshProcess;
	class KProcess *m_proxyProcess;
private:
	bool m_error;
	QCString m_data;
	bool waitsForInput(int type=105);
	bool findLineStartingWith(QCString);
	void sendString(const char*);
	bool gotResponse(const char*);
	char *send0;
	char *send1;
	
	class SSHData {
	public:
		QCString user;
		QCString host;
		QCString port;
		QCString password; //unused right now
		KProcess *newProcess(QObject *parent);
		QString targetData();

	} m_sshData;

	QString m_username;
	QString m_password;

	QString m_sessionName;

	QCString m_sessionID;
	QCString m_sessionCookie;
	QCString m_sessionDisplay;
	QCString m_cookie;

	QString m_sessionString;
	QString m_screenInfo;
	class Private;
	Private *d;

	bool getValue(int id,QCString &val);
	class KProcess *m_Process;

	enum SessionType {
		UnknownSession=0, UnixSession=1, VNCSession=2, WindowsSession=3,LastSession=0xff
	} m_sessionType;


	class  TunneledConnection {
	public:
		TunneledConnection(QDomElement&){}
		virtual ~TunneledConnection(){}
		virtual bool askCredentials(QWidget*,const QString&){return false;}
		virtual QString settingsString() {return "" ;}
	private:
		class Private;
		Private *d;
	} *m_tunneledConnection;



	class TunneledVNCConnection : public TunneledConnection{
	public:
		TunneledVNCConnection(QDomElement &groupEl);
		virtual ~TunneledVNCConnection();
		virtual bool askCredentials(QWidget*,const QString&);
		virtual QString settingsString();
	private:
		QCString password;
		QString host;
		QString display;
		bool remember;
		class Private;
		Private *d;
	};

	class TunneledWindowsConnection : public TunneledConnection{
	public:
		TunneledWindowsConnection(QDomElement &groupEl);
		virtual ~TunneledWindowsConnection();
		virtual bool askCredentials(QWidget*,const QString&);
		virtual QString settingsString();
	private:
		QCString password;
		QString host;
		QString user;
		bool remember;
		class Private;
		Private *d;
	};

	QGuardedPtr<ConnectingDialog> m_connecting;
	bool m_ignoreAbortConnecting;
	class QTimer *m_connectionTimeout;
	void readAdvanced(QDomElement &);
	void readGeneral(QDomElement &);
private  slots:
	void receivedFromSSH (KProcess *proc, char *buffer, int buflen);
	void dataHasBeenWrittenToSSH();	
	void timeoutWhileConnecting();
	void childExited (KProcess*);
signals:
	void proxyStarted();
};

#endif
